Package: MDSvis
Title: Plots of Multi Dimensional Scaling (MDS) results
Version: 1.1.0
Authors@R: 
    c(person(given = "Andrea",
             family = "Vicini",
             role = c("aut"),
             email = "andrea.vicini@uclouvain.be",
             comment = c(ORCID = "0000-0001-9438-6909")),
      person(given = "Philippe",
             family = "Hauchamps",
             role = c("aut", "cre"),
             email = "philippe.hauchamps@uclouvain.be",
             comment = c(ORCID = "0000-0003-2865-1852")),
      person(given = "Shabnam",
             family = "Zaman",
             role = "ctb",
             email = "shabnam.zaman@uclouvain.be",
             comment = c(ORCID = "0000-0001-5399-0130")),
      person(given = "Laurent",
             family = "Gatto",
             role = "aut",
             email = "laurent.gatto@uclouvain.be",
             comment = c(ORCID = "0000-0002-1520-2268")))
Description: This package implements visulization of Multi Dimensional Scaling
             (MDS) results.
License: GPL-3
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
BugReports: https://github.com/UCLouvain-CBIO/MDSvis/issues
URL: https://uclouvain-cbio.github.io/MDSvis
biocViews: FlowCytometry, QualityControl, DimensionReduction,
        MultidimensionalScaling, Software, Visualization
Depends: R (>= 4.6)
Imports: CytoMDS (>= 1.3.5), rlang, ggplot2, plotly, shiny, shinyjs,
        methods
Suggests: knitr, rmarkdown, BiocStyle, HDCytoData, flowCore, testthat
        (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/MDSvis
git_branch: devel
git_last_commit: cd89dfa
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:37:31 UTC; biocbuild
Author: Andrea Vicini [aut] (ORCID: <https://orcid.org/0000-0001-9438-6909>),
  Philippe Hauchamps [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2865-1852>),
  Shabnam Zaman [ctb] (ORCID: <https://orcid.org/0000-0001-5399-0130>),
  Laurent Gatto [aut] (ORCID: <https://orcid.org/0000-0002-1520-2268>)
Maintainer: Philippe Hauchamps <philippe.hauchamps@uclouvain.be>
