# Generated by roxygen2: do not edit by hand

export(mdsvis_app)
import(shiny)
importFrom(CytoMDS,ggplotSampleMDS)
importFrom(CytoMDS,nDim)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,labs)
importFrom(methods,is)
importFrom(plotly,ggplotly)
importFrom(plotly,plotlyOutput)
importFrom(plotly,renderPlotly)
importFrom(rlang,.data)
importFrom(shinyjs,disable)
importFrom(shinyjs,enable)
importFrom(shinyjs,useShinyjs)
