# MDSvis - Copyright (C) <2025-2026>
# <Université catholique de Louvain (UCLouvain), Belgique>
#
#   Description and complete License: see LICENSE file.
#
# This program (MDSvis) is free software:
#   you can redistribute it and/or modify it under the terms of the GNU General
# Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details (<http://www.gnu.org/licenses/>).

#' @title MDSvis package
#' 
#' @name MDSvis
#'
#' @rdname MDSvis
#' 
#' @seealso \link[CytoMDS:CytoMDS-package]{CytoMDS}
#' @seealso [MDSvis::mdsvis_app]
#'
#' @description
#'
#' `MDSvis` provides a shiny application for interactive visualisation of 
#' Multi Dimensional Scaling (MDS) results.

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL
